@@ -1,8 +1,11 @@
+import asyncio
 import datetime
 import json
 import mimetypes
 import os
 
+from dateutil.tz import tzutc
+from fsspec.asyn import sync
 from tornado.web import HTTPError
 
 from s3contents.chunks import (
@@ -26,13 +29,11 @@ from s3contents.ipycompat import (
     validate,
 )
 
-
 DUMMY_CREATED_DATE = datetime.datetime.fromtimestamp(86400)
 NBFORMAT_VERSION = 4
 
 
 class GenericContentsManager(ContentsManager, HasTraits):
-
     # This makes the checkpoints get saved on this directory
     root_dir = Unicode("./", config=True)
 
@@ -73,7 +74,8 @@ class GenericContentsManager(ContentsManager, HasTraits):
     def already_exists(self, path):
         thing = "File" if self.file_exists(path) else "Directory"
         self.do_error(
-            "{thing} already exists: [{path}]".format(thing=thing, path=path), 409
+            "{thing} already exists: [{path}]".format(thing=thing, path=path),
+            409,
         )
 
     def guess_type(self, path, allow_directory=True):
@@ -86,6 +88,9 @@ class GenericContentsManager(ContentsManager, HasTraits):
         ----------
             obj: s3.Object or string
         """
+        self.log.debug(
+            f"guess_type with path={path} and allow_directory={allow_directory}"
+        )
         if path.endswith(".ipynb"):
             return "notebook"
         elif allow_directory and self.dir_exists(path):
@@ -100,17 +105,31 @@ class GenericContentsManager(ContentsManager, HasTraits):
 
     def dir_exists(self, path):
         # Does a directory exist at the given path?
-        self.log.debug("S3contents.GenericManager.dir_exists: path('%s')", path)
+        self.log.debug(
+            "S3contents.GenericManager.dir_exists: path('%s')", path
+        )
         return self.fs.isdir(path)
 
     def get(self, path, content=True, type=None, format=None):
         # Get a file or directory model.
         self.log.debug(
-            "S3contents.GenericManager.get] path('%s') type(%s) format(%s)",
+            "S3contents.GenericManager.get: path('%s') type(%s) format(%s)",
             path,
             type,
             format,
         )
+
+        # This is a hack to remove some startup dialog error from JupyterLab
+        # TODO: Figure out why is this happening
+        if self.parent:
+            if path.startswith(self.parent.root_dir):
+                self.log.debug(
+                    "S3contents.GenericManager.get: removing root_dir (%s) from path",
+                    self.parent.root_dir,
+                )
+                path = path[len(self.parent.root_dir) :]
+        # END hack
+
         path = path.strip("/")
 
         if type is None:
@@ -142,7 +161,9 @@ class GenericContentsManager(ContentsManager, HasTraits):
             content,
             format,
         )
-        return self._notebook_model_from_path(path, content=content, format=format)
+        return self._notebook_model_from_path(
+            path, content=content, format=format
+        )
 
     def _get_file(self, path, content=True, format=None):
         self.log.debug(
@@ -163,25 +184,119 @@ class GenericContentsManager(ContentsManager, HasTraits):
         if self.fs.isdir(path):
             lstat = self.fs.lstat(path)
             if "ST_MTIME" in lstat and lstat["ST_MTIME"]:
-                model["last_modified"] = model["created"] = lstat["ST_MTIME"]
+                model["created"] = model["last_modified"] = lstat["ST_MTIME"]
+
+            self.log.debug(
+                f"s3_detail_to_model={self.convert_s3_details_to_models}"
+                f"dir_s3_detail: path='{path}', lstat={lstat}"
+            )
         if content:
             if not self.dir_exists(path):
                 self.no_such_entity(path)
             model["format"] = "json"
-            dir_content = self.fs.ls(path=path)
-            model["content"] = self._convert_file_records(dir_content)
+            prefixed_path = self.fs.path(path)
+            files_s3_detail = sync(
+                self.fs.fs.loop, self.fs.fs._lsdir, prefixed_path
+            )
+            filtered_files_s3_detail = list(
+                filter(
+                    lambda detail: os.path.basename(detail["Key"])
+                    != self.fs.dir_keep_file
+                    and self.fs.remove_prefix(detail["Key"]) != path,
+                    files_s3_detail,
+                )
+            )
+            filtered_files_s3_detail = sync(
+                self.fs.fs.loop,
+                self.get_content_s3_metadata,
+                filtered_files_s3_detail,
+            )
+            filtered_files_s3_detail = self.filter_deleted_markers(
+                filtered_files_s3_detail
+            )
+            model['content'] = self.convert_s3_details_to_models(
+                filtered_files_s3_detail
+            )
         return model
 
+    def filter_deleted_markers(self, s3_details):
+        def is_delete_marker(detail):
+            return bool("ST_MTIME" in detail and detail["ST_MTIME"])
+
+        return list(filter(is_delete_marker, s3_details))
+
+    async def get_content_s3_metadata(self, s3_details):
+        async def s3_detail_metadata(s3_detail):
+            if s3_detail["StorageClass"] == "DIRECTORY":
+                dir_path = os.path.join(
+                    self.fs.path(s3_detail["Key"]), ".s3keep"
+                )
+                try:
+                    lstat = await self.fs.fs._info(dir_path)
+                    s3_detail['LastModified'] = lstat['LastModified']
+                except FileNotFoundError:
+                    pass
+            st_time = s3_detail.get("LastModified")
+            if st_time:
+                s3_detail["ST_MTIME"] = datetime.datetime(
+                    st_time.year,
+                    st_time.month,
+                    st_time.day,
+                    st_time.hour,
+                    st_time.minute,
+                    st_time.second,
+                    tzinfo=st_time.tzinfo,
+                )
+            return s3_detail
+
+        tasks = [
+            self.fs.fs.loop.create_task(s3_detail_metadata(detail))
+            for detail in s3_details
+        ]
+        details_with_meta = await asyncio.gather(*tasks)
+        return details_with_meta
+
+    def convert_s3_details_to_models(self, s3_details):
+        models = []
+        for s3_detail in s3_details:
+            model_path = s3_detail["Key"]
+            model = base_model(self.fs.remove_prefix(model_path))
+            if s3_detail["StorageClass"] == "DIRECTORY":
+                model["created"] = model["last_modified"] = DUMMY_CREATED_DATE
+                model["type"] = "directory"
+                model['last_modified'] = model['created'] = s3_detail.get(
+                    'LastModified'
+                )
+                if "ST_MTIME" in s3_detail and s3_detail["ST_MTIME"]:
+                    model["created"] = model["last_modified"] = s3_detail[
+                        "ST_MTIME"
+                    ]
+            else:
+                model["last_modified"] = s3_detail.get("LastModified").replace(
+                    microsecond=0, tzinfo=tzutc()
+                )
+                model["created"] = model["last_modified"]
+                model["type"] = (
+                    "notebook" if model_path.endswith(".ipynb") else "file"
+                )
+            models.append(model)
+        return models
+
     def _notebook_model_from_path(self, path, content=False, format=None):
         """
         Build a notebook model from database record.
         """
+        self.log.debug(
+            f"_notebook_model_from_path with path={path}, content={content}, format={format}"
+        )
         model = base_model(path)
         model["type"] = "notebook"
         if self.fs.isfile(path):
-            model["last_modified"] = model["created"] = self.fs.lstat(path)["ST_MTIME"]
+            model["created"] = model["last_modified"] = self.fs.lstat(path)[
+                "ST_MTIME"
+            ]
         else:
-            model["last_modified"] = model["created"] = DUMMY_CREATED_DATE
+            self.do_error("Not Found", 404)
         if content:
             if not self.fs.isfile(path):
                 self.no_such_entity(path)
@@ -197,12 +312,17 @@ class GenericContentsManager(ContentsManager, HasTraits):
         """
         Build a file model from database record.
         """
+        self.log.debug(
+            f"_file_model_from_path with path={path}, content={content}, format={format}"
+        )
         model = base_model(path)
         model["type"] = "file"
         if self.fs.isfile(path):
-            model["last_modified"] = model["created"] = self.fs.lstat(path)["ST_MTIME"]
+            model["created"] = model["last_modified"] = self.fs.lstat(path)[
+                "ST_MTIME"
+            ]
         else:
-            model["last_modified"] = model["created"] = DUMMY_CREATED_DATE
+            model["created"] = model["last_modified"] = DUMMY_CREATED_DATE
         if content:
             try:
                 # Get updated format from fs.read()
@@ -216,36 +336,18 @@ class GenericContentsManager(ContentsManager, HasTraits):
             model["mimetype"] = mimetypes.guess_type(path)[0] or "text/plain"
         return model
 
-    def _convert_file_records(self, paths):
-        """
-        Applies _notebook_model_from_s3_path or _file_model_from_s3_path to each entry of `paths`,
-        depending on the result of `guess_type`.
-        """
-        ret = []
-        for path in paths:
-            # path = self.fs.remove_prefix(path, self.prefix)  # Remove bucket prefix from paths
-            if os.path.basename(path) == self.fs.dir_keep_file:
-                continue
-            type_ = self.guess_type(path, allow_directory=True)
-            if type_ == "notebook":
-                ret.append(self._notebook_model_from_path(path, False))
-            elif type_ == "file":
-                ret.append(self._file_model_from_path(path, False, None))
-            elif type_ == "directory":
-                ret.append(self._directory_model_from_path(path, False))
-            else:
-                self.do_error("Unknown file type %s for file '%s'" % (type_, path), 500)
-        return ret
-
     def save(self, model, path):
-        """Save a file or directory model to path.
-        """
+        """Save a file or directory model to path."""
+
+        self.log.debug(f"save with path={path}, model={model}")
 
         # Chunked uploads
         # See https://jupyter-notebook.readthedocs.io/en/stable/extending/contents.html#chunked-saving
         chunk = model.get("chunk", None)
         if chunk is not None:
-            return self._save_large_file(chunk, model, path, model.get("format"))
+            return self._save_large_file(
+                chunk, model, path, model.get("format")
+            )
 
         self.log.debug("S3contents.GenericManager.save %s: '%s'", model, path)
         if "type" not in model:
@@ -266,8 +368,12 @@ class GenericContentsManager(ContentsManager, HasTraits):
             else:
                 validation_message = self._save_directory(path)
         except Exception as e:
-            self.log.error("Error while saving file: %s %s", path, e, exc_info=True)
-            self.do_error("Unexpected error while saving file: %s %s" % (path, e), 500)
+            self.log.error(
+                "Error while saving file: %s %s", path, e, exc_info=True
+            )
+            self.do_error(
+                "Unexpected error while saving file: %s %s" % (path, e), 500
+            )
 
         model = self.get(path, type=model["type"], content=False)
 
@@ -292,13 +398,17 @@ class GenericContentsManager(ContentsManager, HasTraits):
 
         if format not in {"text", "base64"}:
             self.do_error(
-                "Must specify format of file contents as 'text' or 'base64'", 400
+                "Must specify format of file contents as 'text' or 'base64'",
+                400,
             )
 
         prune_stale_chunks()
 
         self.log.debug(
-            "S3contents.GenericManager.save (chunk %s) %s: '%s'", chunk, model, path
+            "S3contents.GenericManager.save (chunk %s) %s: '%s'",
+            chunk,
+            model,
+            path,
         )
 
         try:
@@ -327,7 +437,8 @@ class GenericContentsManager(ContentsManager, HasTraits):
         nb_contents = from_dict(model["content"])
         self.check_and_sign(nb_contents, path)
         file_contents = json.dumps(model["content"])
-        self.fs.write(path, file_contents)
+        file_format = model.get("format")
+        self.fs.write(path, file_contents, file_format)
         self.validate_notebook_model(model)
         return model.get("message")
 
@@ -363,8 +474,7 @@ class GenericContentsManager(ContentsManager, HasTraits):
             self.no_such_entity(old_path)
 
     def delete_file(self, path):
-        """Delete the file or directory at path.
-        """
+        """Delete the file or directory at path."""
         self.log.debug("S3contents.GenericManager.delete_file '%s'", path)
         if self.file_exists(path) or self.dir_exists(path):
             self.fs.rm(path)
@@ -372,8 +482,7 @@ class GenericContentsManager(ContentsManager, HasTraits):
             self.no_such_entity(path)
 
     def is_hidden(self, path):
-        """Is path a hidden directory or file?
-        """
+        """Is path a hidden directory or file?"""
         self.log.debug("S3contents.GenericManager.is_hidden '%s'", path)
         return False
 
@@ -391,11 +500,16 @@ class GenericContentsManager(ContentsManager, HasTraits):
         if self.post_save_hook:
             try:
                 self.log.debug("Running post-save hook on %s", s3_path)
-                self.post_save_hook(s3_path=s3_path, model=model, contents_manager=self)
+                self.post_save_hook(
+                    s3_path=s3_path, model=model, contents_manager=self
+                )
             except Exception as e:
-                self.log.error("Post-save hook failed o-n %s", s3_path, exc_info=True)
+                self.log.error(
+                    "Post-save hook failed o-n %s", s3_path, exc_info=True
+                )
                 raise HTTPError(
-                    500, "Unexpected error while running post hook save: %s" % e
+                    500,
+                    "Unexpected error while running post hook save: %s" % e,
                 ) from e
 
 
@@ -415,6 +529,8 @@ def base_model(path):
 def base_directory_model(path):
     model = base_model(path)
     model.update(
-        type="directory", last_modified=DUMMY_CREATED_DATE, created=DUMMY_CREATED_DATE,
+        type="directory",
+        last_modified=DUMMY_CREATED_DATE,
+        created=DUMMY_CREATED_DATE,
     )
     return model
