@@ -1,10 +1,12 @@
 import time
 
 import pytest
+from notebook.services.contents.tests.test_largefilemanager import (
+    TestLargeFileManager,
+)
 
 from s3contents import S3ContentsManager
 from s3contents.chunks import content_chunks
-from s3contents.ipycompat import TestLargeFileManager
 
 
 @pytest.mark.minio
@@ -34,7 +36,8 @@ class S3ContentsManagerLargeFileTestCase(TestLargeFileManager):
 
     def make_dir(self, api_path):
         self.contents_manager.new(
-            model={"type": "directory"}, path=api_path,
+            model={"type": "directory"},
+            path=api_path,
         )
 
     def test_save(self):
@@ -49,5 +52,5 @@ class S3ContentsManagerLargeFileTestCase(TestLargeFileManager):
         self.assertNotIn("stale_file.txt", current_value)
 
 
-# This needs to be removed or else we'll run the main IPython tests as well.
+# Needs to be removed or else we'll run the main IPython tests as well
 del TestLargeFileManager
